%-------------------------------------------------------------------------- 
%  Mathematische Modellierung mit MATLAB, F. Hausser und Y. Luchko
%  Basketball Freiwurf
%     
%  function  [thetaMinMax, vMinMax = fehlerWurfbahn(theta0,v0,hS)
%     Eingabeparameter:
%        theta0:     Abwurfwinkel [grad]
%        v0:         Abwurfgeschwindigkeit [m]
%        hS:         Groesse des Werfers   [m]
%     Ausgabeparameter:
%        thetaMinMax: minimaler und maximaler Abwurfwinkel, bei fester 
%                     Abwurfgeschwindigkeit v0, so dass Ball noch im Korb
%        vMinMax:     minimale und maximale Abwurfgeschwindigkeit, bei
%                     festem Abwurfwinkel theta0, so dass Ball noch im Korb
%                                     
%     Author: Frank Hausser
%     Date:   8/2009
%
%--------------------------------------------------------------------------


function [thetaMinMax,vMinMax] = fehlerWurfbahn(theta0Grad,v0,hS)

%--------------------  Parameter ------------------------------------------
if ( nargin < 3)
    hS = 2.13; % Groesse von Dirk Nowitzky
end 

parameter = getParameter();
l  = parameter.abstandHorizontal;          % Horizontale Entfernung [m]
h  = parameter.hoeheKorb - 5/4 * hS;       % vertikale Entfernung [m]
Rr = parameter.radiusRing;                 % Ringradius
Rb = parameter.radiusBall;                 % Ballradius
g  = parameter.erdbeschleunigung;          % Erdbeschleunigung

%-----------Check, ob Anfangsdaten zulaessig -----------------------------

if zulaessigerPunkt(theta0Grad,v0) == false
   thetaMinMax = [];
   vMinMax = [];
   return;
end

%---------- Bestimmung von thetaMin ---------------------------------------
% suche ersten unzulaessigen Winkel auf grobem Raster
thetaMin = theta0Grad - 0.5;
while( zulaessigerPunkt(thetaMin,v0)  ) 
    thetaMin = thetaMin - 0.5;
end

% Bisektion zur Feinbestimmung
links = thetaMin;
rechts = links + 0.5;
while abs(rechts - links) > 1.e-8
   mitte = (rechts + links)/2;
   if( zulaessigerPunkt(mitte,v0) )
       rechts = mitte;
   else
       links = mitte;
   end
end
thetaMin = mitte;

%---------- Bestimmung von thetaMax ---------------------------------------
% suche ersten unzulaessigen Winkel auf grobem Raster
thetaMax = theta0Grad + 0.5;
while( zulaessigerPunkt(thetaMax,v0)  ) 
    thetaMax = thetaMax + 0.5;
end

% Bisektion zur Feinbestimmung
rechts = thetaMax;
links  = thetaMax - 0.5;
while abs(rechts - links) > 1.e-8
   mitte = (rechts + links)/2;
   if( zulaessigerPunkt(mitte,v0) )
       links = mitte;
   else
       rechts = mitte;
   end
end
thetaMax = mitte;

%--------------------------------------------------------------------------
thetaMinMax = [thetaMin, thetaMax];   %Rueckgabewert setzen

%---------- Bestimmung von vMin -------------------------------------------
% suche erste unzulaessige Geschwindigkeit auf grobem Raster
vMin = v0 - 0.1;
while( zulaessigerPunkt(theta0Grad,vMin)  ) 
    vMin = vMin - 0.1;
end

% Bisektion zur Feinbestimmung
links = vMin;
rechts = links + 0.1;
while abs(rechts - links) > 1.e-8
   mitte = (rechts + links)/2;
   if( zulaessigerPunkt(theta0Grad,mitte) )
       rechts = mitte;
   else
       links = mitte;
   end
end
vMin = mitte;

%---------- Bestimmung von vMax ---------------------------------------
% suche erste  unzulaessige Geschwindigket auf grobem Raster
vMax = v0 + 0.1;
while( zulaessigerPunkt(theta0Grad,vMax)  ) 
    vMax = vMax + 0.1;
end

% Bisektion zur Feinbestimmung
rechts = vMax;
links  = vMax - 0.1;
while abs(rechts - links) > 1.e-8
   mitte = (rechts + links)/2;
   if( zulaessigerPunkt(theta0Grad,mitte) )
       links = mitte;
   else
       rechts = mitte;
   end
end
vMax = mitte;

%--------------------------------------------------------------------------
vMinMax = [vMin, vMax];    % Rueckgabewert setzen 


%---------- Hilfsfunktion (nested function) --------------------------------
%   Test ob ein Abwurfwinkel noch zu erfolgreichem Korb fuehrt

function zulaessig = zulaessigerPunkt(thetaGrad,v)
    theta = thetaGrad*pi/180;
    vx = v * cos(theta);
    vy = v * sin(theta);
    % Schranken fuer zu untersuchende Zeitspanne
    T = (vy + sqrt(vy*vy - 2*g*h))/g; % Zeit bis Ringhoehe in vertikaler
                                         % Richtung von oben erreicht
    t0 = (l - Rr - Rb)/vx; % Zeit bis der Ballmittelpunkt in horizontaler 
                            % Richtung nur noch den Abstand Radius Ball hat
                           
    % Berechne Trajektorie (Wurfparabel) analytisch 
    t = linspace(t0,T,100);
    x = vx*t;
    y = vy*t - g*t.*t/2;
    
    zulaessig = true;
    
    % Teste, ob Ball den Ring berhrt
    if( x(end) + Rb > l + Rr )      % hinterer Rand des Rings
        zulaessig = false; 
    elseif ( x(end) - Rb < l - Rr ) % vorderer Rand des Rings
        zulaessig = false;          % erster Schnelltest fuer x(T) 
    else                            % vorderer Rand des Rings
        xs = x - l + Rr;  % horizontaler Abstand des Ballmittelpunktes 
        ys = y - h;       % vertikaler Abstand des Ballmittelpunktes
        s2 = xs.*xs + ys.*ys; 
        if min(s2) < Rb*Rb 
            zulaessig = false;
        end    
    end
end

end