function [x,nIt] = gaussNewton(x0,fun,Jfun,tol,nItMax, method)
% [x,nIt] = gaussNewton(x0,fun,Jfun,tol,nItMax)
%  Loesung des nichtlinearen Ausgleichgsproblems norm(f(x)),2) = min
%  mit dem Gauss-Newton-Verfahren; die linearen Ausgleichsprobleme werden
%  mit der QR-Zerlegung geloest
% Input: 
%   x0:     Startvektor 
%   fun:    function handle fuer Funktion f
%   Jfun:   function handle fuer Jacobimatrix von f
%   tol:    Toleranz, Abbruch, wenn norm(Jfun(x)^T*fun(x),2) < tol 
%   nItMax: Maximale Iterationszahl   
%   method: Berechnungsmethode zur Loesung des linearen Ausgleichusprobleme;
%           1 : Normalgleichung; 2 : QR-Zerlegung
%
% Output:
%   x:      approximative Loesung (Minimalstelle)
%   nIt:    Anzahl benoetigter Iterationen


%% --------------------------- Initialisierungen --------------------------
n     = length(x0);
nIt   = 0; 
x     = x0;

if nargin < 6
    method = 2;
end
if nargin < 5
    nItMax = 100;
end
if nargin < 4
    tol = 1.e-6;
end

res   = tol+1;

%% -------- Loesen einer Folge von linearen Ausgleichsproblemen -----------
while res > tol
    if nIt >= nItMax
        disp(' Toleranz nicht erreicht, nach der maximalen Anzahl von Iterationen ');
        disp(' Fehler:  '),    disp(res);
        disp(' Toleranz:  '),  disp(tol);
        break
    end
    
    Fx  = fun(x);
    DFx = Jfun(x);
    b    = -DFx'*Fx;  
    res  = norm(b);      % Norm des Gradienten des Zielfunktionals
    
    
    if method == 1
    %--- Loesung mit Normalgleichungen ----   
        deltax = (DFx'*DFx)\b; 
    else
    %--- Loesung mit QR-Rerlegung ---
    [Q,R]     = qr(DFx,0);
    c         = -Q'*fun(x);
    
    deltax    = zeros(n,1);
    deltax(n) = c(n)/R(n,n); 
        for j = n-1:-1:1
            deltax(j) = ( c(j) - R(j,(j+1):n) * deltax(j+1:n) ) / R(j,j); 
        end;
    end
    
    x      = x + deltax;
    min    = norm(Fx,2);
    nIt    = nIt + 1;
end

disp('Summe der Fehlerquadrate:   '), disp(min);