%--------------------------------------------------------------------------   
%     Mathematische Modellierung mit MATLAB, F. Hausser und Y. Luchko
%     Schwingungen eines ebenen Fachwerkes,  Listing 6.5, 6.7, 6.10
%     Anwendungsskript, benoetigt die Funktionen
%              S = assembleSteifigkeitsMatrix(A,p,nDirichlet)
%              M = assembleMassenMatrix(A,p,nDirichlet)
%              xdt = fmm(t,x)
%              xdt = fmmLinear(t,x)
%              plotVerschiebung(A,p,u)
%              plotVerschiebungMitRuhelage(A,p,u)
%              plotVergleichVerschiebung(uNonLin,uLin)
%              showMovieFMM(A,t,z) 
%              
%     Author: Frank Hausser
%     Date:   4/2010
%--------------------------------------------------------------------------

clear all;
close all;

%------------------ Deklaration globaler Variablen ------------------------

% globale Variablen in den Funktionen
%         function xdt = fmm(t,x)        (nichtlineares Anfangswertproblem)
%         function xdt = fmmLinear(t,x)  (lineares Anfangswertproblem)
%  die in diesem Skript gesetzt werden

global A;     % Adjazenzmatrix
global Ag;    % gewichtete Adjazenzmatrix 
global S;     % Steifigkeitsmatrix
global M;     % Massenmatrix
global m;     % Massenvektor
global p;     % Positionsvektor im lastfreien Gleichgewicht
global w;     % Winkelgeschwindigkeit der aeusseren Kraft
global sv;    % Amplitute der auesseren Kraft

sv = zeros(8,1);
w = 0;

%---------------- Adjazenzmatrix und Knotenvektor (Listing 6.5) -----------

% Positionen der  6 Knoten in der Ruhelage
p = [ 1 0 2 0 1 1 2 1 0 0 3 0]'; 

%Adjazenzmatrix (eine Zeile pro Knoten)
A = [0 1 1 1 1 0; 
     1 0 1 1 0 1; 
     1 1 0 1 1 0; 
     1 1 1 0 0 1;
     1 0 1 0 0 0; 
     0 1 0 1 0 0]; 

%Adjazenzmatrix (gewichtet mit Ruhelngen der Federn)
a  = 1;          % horizontaler Balken hat Laenge 1
b = sqrt(2);

Ag = [0 a a b a 0; 
     a 0 b a 0 a; 
     a b 0 a b 0; 
     b a a 0 0 b ]; 
 
%-------- Assemblierung der Steifigkeits- und Massenmatrix ----------------
 
S = assembleSteifigkeitsMatrix(A,p,2);
M = assembleMassenMatrix(A,p,2);
m = diag(M);
m1 = m.^(-1);
MInv = diag(m1);

%----------- Eigenschwingungen  (Listing 6.7) -----------------------------
[V,L] = eig(MInv*S);
winkelFrequenzen = sqrt(diag(L));
eigenFrequenzen  = winkelFrequenzen/2/pi;
mode = 1;                             % 1. Eigenschwingung wird ausgewaehlt
ev = V(:,mode)
plotVerschiebung(A,p,0.5*ev);
disp('weiter mit <return>');
pause  % weiter mit <return>

% Amplituden aller Eigenschwingungen ( vergleiche Abb. 6.5 )
for mode = 1:8
    ev = V(:,mode)
    plotVerschiebungMitRuhelage(A,p,0.5*ev);
    pause
    disp('weiter mit <return>');
end


%---------- Gleichgewicht unter Last (Listing 6.10)  -----------------------   
fLast    = zeros(8,1);   % Lastvektor
fLast(2) = -0.2;         % an Knoten 1 wirkt Kraft in vertikaler Richtung

% Lineares Modell
uLin = S\fLast

% Nichtlineares Modell
x0 = p(1:8);
v0 = zeros(8,1);
z0 = [x0;v0];
fLastNonLin = [v0;fLast./m];

fNonLin = @(z) ( fmm(0,z) + fLastNonLin );
z = fsolve(fNonLin,z0);
uNonLin = z(1:8) - x0

plotVergleichVerschiebung(uNonLin,uLin);  % Erzeugt Abbildung 6.8 
pause      % weiter mit <return>
disp('weiter mit <return>');

%--------------------------------------------------------------------------
%------- Zeitabhngige Loesung (Abbildung 6.6, 6.7) ------------------------
%------- Vergleich des linearisierten und nichtlinearen Modells -----------


%---- Berechnung der Loesung ohne aeussere Kraft (Abb. 6.6) ---------------
mode = 1;
w0   = winkelFrequenzen(mode);
tIntervall = [0,10*pi/w0]; % Zeitintervall

% Anfangsbedingungen: Auslenkung in Richtung 1.Eigenschwingung
x0   = p(1:8) + 0.5*V(:,mode);
v0   = zeros(8,1);
z0   = [x0;v0];

% aeussere Kraft F_L = 0
w  = 0;  % globale Variable 

% Numerische Loesung des Anfangswertproblems
options = odeset('RelTol',1e-6,'AbsTol',1e-6);
[tNLin,zNLin]= ode45(@fmm,       tIntervall, z0, options); 
[tLin,zLin]  = ode45(@fmmLinear, tIntervall, z0, options); 

% Plot der Auslenkung des 1. Knotens, % x-Koordinate
figure(1)  
plot(tNLin,zNLin(:,1)-1,'k-', tLin,zLin(:,1)-1,'k--','linewidth',2);
legend('nichtlinear','linear');
xlabel('Zeit t')
ylabel('Verschiebung u_1 = x_1 - 1');
pbaspect([2,1,1]);

% Plot der Auslenkung des 1. Knotens, % y-Koordinate
figure(2) % y-Koordinate
plot(tNLin,zNLin(:,2),'k', tLin,zLin(:,2),'k--','linewidth',2);
xlabel('Zeit t')
ylabel('Verschiebung u_2 = y_1 - 1');
legend('nichtlinear','linear');
pbaspect([2,1,1]);

pause
disp('weiter mit <return>');


%---- Berechnung der Loesung bei period. Angeregung (Abb. 6.7) ------------

tIntervall   = [0,10*pi/w0];

% Anfangsbedingungen
x0 = p(1:8);
v0 = zeros(8,1);
z0 = [x0;v0];

% aeussere Kraft F_L: periodisch in Richtung 1.Eigenschwingung
mode = 1;
w = winkelFrequenzen(mode);   % globale Variable
sv = 0.1*V(:,mode);           % globale Variable

% Numerische Loesung des Anfangswertproblems
options = odeset('RelTol',1e-6,'AbsTol',1e-6);
[tNLin,zNLin]= ode45(@fmm,       tIntervall, z0, options); 
[tLin,zLin]  = ode45(@fmmLinear, tIntervall, z0, options); 

% Plot der Auslenkung des 1. Knotens, % x-Koordinate
figure(1)
plot(tNLin,zNLin(:,1)-1,'k-', tLin,zLin(:,1)-1,'k--','linewidth',2);
legend('nichtlinear','linear');
xlabel('Zeit t')
ylabel('Verschiebung u_1 = x_1 - 1');
pbaspect([2,1,1]);

% Plot der Auslenkung des 2. Knotens, % y-Koordinate
figure(2)
plot(tNLin,zNLin(:,2),'k', tLin,zLin(:,2),'k--','linewidth',2);
xlabel('Zeit t')
ylabel('Verschiebung u_2 = y_1 - 1');
legend('nichtlinear','linear');
pbaspect([2,1,1]);

pause
disp('weiter mit <return>');

%  Animation der Loesungen:  
close all;
% benoetige die lsung an quidistanten Zeitpunkten:
tIntervall = linspace(0,20*pi/w0,200);
options = odeset('RelTol',1e-6,'AbsTol',1e-6);
[tNonLin,zNonLin]= ode45(@fmm, tIntervall, z0, options); 
[tLin,zLin]= ode45(@fmmLinear, tIntervall, z0, options); 

% nichtlinear
showMovieFMM(A,tNonLin,zNonLin);
pause
disp('weiter mit <return>');
 
% linearisiert 
showMovieFMM(A,tLin,zLin);

