%--------------------------------------------------------------------------
%  Mathematische Modellierung mit MATLAB, F. Hausser und Y. Luchko
%  Kapitel 8, Abbildung 8.3:
%       Zeitliche Entwicklung der Fischbestnde 
%       mit den Fangquoten $c = 0.15$, $c = 0.25$ und  $c = 0.4$  
%
%     Author: Yury Luchko
%     Date:   3/2010
%--------------------------------------------------------------------------
clear

% 1. Fall: Fangquote c = 0.15

% Systemparameter:
% Laenge des Zeitintervalls
L=8.0;
% Fangmenge pro Zeiteinheit
c = 0.15;
% Gleichgewichtslsungen
A = (1 - sqrt(1 - 4*c))/2;
B = (1 + sqrt(1 - 4*c))/2;
x = 0:0.1:L;
y = 0:0.1:L;
y(:)=A;
subplot(1,3,1)
plot(x,y,'--','Color','k','linewidth',2);
hold on;
y(:) = B;
plot(x,y,'-.','Color','k','linewidth',2);
legend('Gleichgewichtslsung n_1',... 
  'Gleichgewichtslsung n_2');
% Rechte Seite der DGL
func = @(t,n) (n(1)*(1-n(1))-c);
options = odeset('AbsTol',1.e-8,'RelTol',1.e-4,'Refine',20);
% 1. Anfangsbedingung (groesser als die Gleichgewichtsloesung B)
n0 = B+1;
% Numerische Lsung der DGL
[t,n] = ode45(func,[0 L],n0,options);
plot(t,n(:),'k','linewidth',2)
% 2. Anfangsbedingung (zwischen den Gleichgewichtslsungen A und B)
n0 = 0.05*(B-A) + A;
% Numerische Lsung der DGL
[t,n] = ode45(func,[0 L],n0,options);
plot(t,n(:),'k','linewidth',2)
% 3. Anfangsbedingung (kleiner als die Gleichgewichtsloesung A)
n0 = 0.95*A;
% Numerische Lsung der DGL
[t,n] = ode45(func,[0 L],n0);
plot(t,n(:),'k','linewidth',2)
axis([0,L,0,1.8])
ylabel('n(t)')
xlabel('t')
hold off;

% 2. Fall: Fangquote c = 0.25

% Systemparameter:
% Laenge des Zeitintervalls
L=15.0;
% Fangmenge pro Zeiteinheit
c = 0.25;
% Gleichgewichtslsung
A =  1/2;
x = 0:0.1:L;
y = 0:0.1:L;
y(:)=A;
subplot(1,3,2)
plot(x,y,'--','Color','k','linewidth',2);
hold on;
legend('Gleichgewichtslsung n_1 = n_2');
% Rechte Seite der DG
func1 = @(t,n) (n(1)*(1-n(1))-c);
% 1. Anfangsbedingung (groesser als die Gleichgewichtsloesung)
n0 = A+1;
% Numerische Lsung der DGL
[t,n] = ode45(func1,[0 L],n0);
plot(t,n(:),'k','linewidth',2)
% 2. Anfangsbedingung (kleiner als die Gleichgewichtsloesung)
n0 = 0.9*A;
% Numerische Lsung der DGL
[t,n] = ode45(func1,[0 L],n0);
plot(t,n(:),'k','linewidth',2)
ylabel('n(t)')
xlabel('t')
hold off;

% 3. Fall: Fangquote c = 0.4

% Systemparameter:
% Laenge des Zeitintervalls
L=6.0;
% Fangmenge pro Zeiteinheit
c = 0.4;
hold on;
% Rechte Seite der DGL
func2 = @(t,n) (n(1)*(1-n(1))-c);
% 1. Anfangsbedingung 
n0 = 3;
% Numerische Lsung der DGL
[t,n] = ode45(func2,[0 L],n0);
subplot(1,3,3)
plot(t,n(:),'k','linewidth',2)
hold on;
% 2. Anfangsbedingung 
n0 = 2;
% Numerische Lsung der DGL
[t,n] = ode45(func2,[0 L],n0);
plot(t,n(:),'k','linewidth',2)
% 3. Anfangsbedingung 
n0 = 1;
% Numerische Lsung der DGL
[t,n] = ode45(func2,[0 L],n0);
plot(t,n(:),'k','linewidth',2)
axis([0,L,0,3.0])
ylabel('n(t)')
xlabel('t')
hold off;